@@ -116,14 +116,71 @@ def on_sourceview_event_after_button_press(dad, text_view, event):
         text_buffer.place_cursor(text_iter)
     return False
 
+def on_sourceview_list_change_level(dad, iter_insert, list_info, text_buffer, level_increase):
+    """Called at list indent/unindent time"""
+    dad.user_active = False
+    end_offset = dad.lists_handler.get_multiline_list_element_end_offset(iter_insert, list_info)
+    curr_offset = list_info["startoffs"]
+    curr_level = list_info["level"]
+    next_level = curr_level+1 if level_increase else curr_level-1
+    iter_start = text_buffer.get_iter_at_offset(curr_offset)
+    prev_list_info = dad.lists_handler.get_prev_list_info_on_level(iter_start, next_level)
+    #print prev_list_info
+    if list_info["num"] != 0:
+        bull_offset = curr_offset + 3*list_info["level"]
+        if list_info["num"] < 0:
+            if prev_list_info != None and prev_list_info["num"] < 0:
+                bull_idx = prev_list_info["num"]*(-1) - 1
+            else:
+                idx_old = list_info["num"]*(-1) - 1
+                idx_offset = idx_old - curr_level % cons.NUM_CHARS_LISTBUL
+                bull_idx = (next_level + idx_offset) % cons.NUM_CHARS_LISTBUL
+            dad.replace_text_at_offset(cons.CHARS_LISTBUL[bull_idx],
+                bull_offset, bull_offset+1, text_buffer)
+        else:
+            idx = list_info["aux"]
+            if prev_list_info != None and prev_list_info["num"] > 0:
+                this_num = prev_list_info["num"] + 1
+                index = prev_list_info["aux"]
+            else:
+                this_num = 1
+                idx_old = list_info["aux"]
+                idx_offset = idx_old - curr_level % cons.NUM_CHARS_LISTNUM
+                index = (next_level + idx_offset) % cons.NUM_CHARS_LISTNUM
+            text_to = str(this_num) + cons.CHARS_LISTNUM[index] + cons.CHAR_SPACE
+            dad.replace_text_at_offset(text_to, bull_offset,
+                bull_offset+dad.lists_handler.get_leading_chars_num(list_info["num"]), text_buffer)
+    iter_start = text_buffer.get_iter_at_offset(curr_offset)
+    #print "%s -> %s" % (curr_offset, end_offset)
+    while curr_offset < end_offset:
+        if level_increase:
+            text_buffer.insert(iter_start, 3*cons.CHAR_SPACE)
+            end_offset += 3
+            iter_start = text_buffer.get_iter_at_offset(curr_offset+3)
+        else:
+            text_buffer.delete(iter_start, text_buffer.get_iter_at_offset(curr_offset+3))
+            end_offset -= 3
+            iter_start = text_buffer.get_iter_at_offset(curr_offset+1)
+        if not dad.lists_handler.char_iter_forward_to_newline(iter_start) or not iter_start.forward_char():
+            break
+        curr_offset = iter_start.get_offset()
+    dad.user_active = True
+    dad.state_machine.update_state()
+
 def on_sourceview_event_after_key_press(dad, text_view, event):
     """Called after every gtk.gdk.KEY_PRESS on the SourceView"""
     text_buffer = text_view.get_buffer()
     keyname = gtk.gdk.keyval_name(event.keyval)
     if (event.state & gtk.gdk.SHIFT_MASK):
         if keyname == cons.STR_KEY_RETURN:
-            text_buffer.insert(text_buffer.get_iter_at_mark(text_buffer.get_insert()), 3*cons.CHAR_SPACE)
-    elif keyname in [cons.STR_KEY_RETURN, cons.STR_KEY_SPACE, cons.STR_KEY_TAB]:
+            iter_insert = text_buffer.get_iter_at_mark(text_buffer.get_insert())
+            if not iter_insert: return False
+            iter_start = iter_insert.copy()
+            iter_start.backward_char()
+            list_info = dad.lists_handler.get_paragraph_list_info(iter_start)
+            if list_info:
+                text_buffer.insert(text_buffer.get_iter_at_mark(text_buffer.get_insert()), 3*(1+list_info["level"])*cons.CHAR_SPACE)
+    elif keyname in [cons.STR_KEY_RETURN, cons.STR_KEY_SPACE]:
         iter_insert = text_buffer.get_iter_at_mark(text_buffer.get_insert())
         if not iter_insert: return False
         iter_start = iter_insert.copy()
@@ -138,40 +195,69 @@ def on_sourceview_event_after_key_press(dad, text_view, event):
             if iter_start.backward_char() and iter_start.get_char() == cons.CHAR_NEWLINE:
                 return False # former was an empty row
             list_info = dad.lists_handler.get_paragraph_list_info(iter_start)
-            if list_info[0] == None:
+            if not list_info:
                 if dad.auto_indent:
                     iter_start = iter_insert.copy()
                     former_line_indent = get_former_line_indentation(iter_start)
                     if former_line_indent: text_buffer.insert_at_cursor(former_line_indent)
                 return False # former was not a list
-            # possible list quit
-            iter_list_quit = iter_insert.copy()
-            if (list_info[0] == 0 and iter_list_quit.backward_chars(3) and iter_list_quit.get_char() == cons.CHAR_LISTBUL):
+            # possible enter on empty list element
+            insert_offset = iter_insert.get_offset()
+            chars_to_startoffs = 1 + dad.lists_handler.get_leading_chars_num(list_info["num"]) + 3*list_info["level"]
+            if (insert_offset - list_info["startoffs"]) == chars_to_startoffs:
+                # enter on empty list element
+                if list_info["level"] > 0:
+                    on_sourceview_list_change_level(dad, iter_insert, list_info, text_buffer, False)
+                    iter_insert = text_buffer.get_iter_at_mark(text_buffer.get_insert())
+                    iter_list_quit = text_buffer.get_iter_at_offset(iter_insert.get_offset()-1)
+                else:
+                    iter_list_quit = text_buffer.get_iter_at_offset(list_info["startoffs"])
                 text_buffer.delete(iter_list_quit, iter_insert)
-                return False # former was an empty paragraph => list quit
-            elif (list_info[0] == -1 and iter_list_quit.backward_chars(3) and iter_list_quit.get_char() in [cons.CHAR_LISTTODO, cons.CHAR_LISTDONEOK, cons.CHAR_LISTDONEFAIL]):
-                text_buffer.delete(iter_list_quit, iter_insert)
-                return False # former was an empty paragraph => list quit
-            elif (list_info[0] > 0 and iter_list_quit.backward_chars(2) and iter_list_quit.get_char() == cons.CHAR_SPACE\
-            and iter_list_quit.backward_char() and iter_list_quit.get_char() == '.'):
-                iter_list_quit.backward_chars(len(str(list_info[0])))
-                text_buffer.delete(iter_list_quit, iter_insert)
-                return False # former was an empty paragraph => list quit
-            if list_info[0] == 0: text_buffer.insert(iter_insert, cons.CHAR_LISTBUL + cons.CHAR_SPACE)
-            elif list_info[0] == -1: text_buffer.insert(iter_insert, cons.CHAR_LISTTODO + cons.CHAR_SPACE)
-            else: text_buffer.insert(iter_insert, '%s. ' % (list_info[0] + 1))
-        elif keyname == cons.STR_KEY_TAB:
-            list_info = dad.lists_handler.get_paragraph_list_info(iter_start)
-            if list_info != None:
-                print list_info
+                return False
+            # list new element
+            curr_level = list_info["level"]
+            pre_spaces = 3*curr_level*cons.CHAR_SPACE if curr_level else ""
+            if list_info["num"] < 0:
+                index = list_info["num"]*(-1) - 1
+                text_buffer.insert(iter_insert, pre_spaces+cons.CHARS_LISTBUL[index]+cons.CHAR_SPACE)
+            elif list_info["num"] == 0:
+                text_buffer.insert(iter_insert, pre_spaces+cons.CHAR_LISTTODO+cons.CHAR_SPACE)
+            else:
+                new_num = list_info["num"] + 1
+                index = list_info["aux"]
+                text_buffer.insert(iter_insert, pre_spaces + str(new_num) + cons.CHARS_LISTNUM[index] + cons.CHAR_SPACE)
+                new_num += 1
+                iter_start = text_buffer.get_iter_at_offset(insert_offset)
+                dad.lists_handler.char_iter_forward_to_newline(iter_start)
+                list_info = dad.lists_handler.get_next_list_info_on_level(iter_start, curr_level)
+                #print list_info
+                while list_info and list_info["num"] > 0:
+                    iter_start = text_buffer.get_iter_at_offset(list_info["startoffs"])
+                    end_offset = dad.lists_handler.get_multiline_list_element_end_offset(iter_start, list_info)
+                    iter_end = text_buffer.get_iter_at_offset(end_offset)
+                    iter_start, iter_end, chars_rm = dad.lists_handler.list_check_n_remove_old_list_type_leading(iter_start, iter_end, text_buffer)
+                    end_offset -= chars_rm
+                    text_buffer.insert(iter_start, str(new_num) + cons.CHARS_LISTNUM[index] + cons.CHAR_SPACE)
+                    end_offset += dad.lists_handler.get_leading_chars_num(new_num)
+                    iter_start = text_buffer.get_iter_at_offset(end_offset)
+                    new_num += 1
+                    list_info = dad.lists_handler.get_next_list_info_on_level(iter_start, curr_level)
         elif keyname == cons.STR_KEY_SPACE:
             if iter_start.backward_chars(2):
-                if iter_start.get_char() == cons.CHAR_GREATER and iter_start.backward_char()\
-                and iter_start.get_char() == cons.CHAR_MINUS and iter_start.backward_char():
-                    if iter_start.get_char() == cons.CHAR_LESSER:
-                        dad.special_char_replace(cons.SPECIAL_CHAR_ARROW_DOUBLE, iter_start, iter_insert, text_buffer)
-                    elif iter_start.get_char() == cons.CHAR_MINUS:
-                        dad.special_char_replace(cons.SPECIAL_CHAR_ARROW_RIGHT, iter_start, iter_insert, text_buffer)
+                if iter_start.get_char() == cons.CHAR_GREATER and iter_start.backward_char():
+                    # Start bulleted list on "<> ", "-> " and "=> " at line start
+                    if iter_start.get_line_offset() == 0:
+                        if iter_start.get_char() == cons.CHAR_LESSER:
+                            dad.special_char_replace(cons.CHARS_LISTBUL[1], iter_start, iter_insert, text_buffer)
+                        elif iter_start.get_char() == cons.CHAR_MINUS:
+                            dad.special_char_replace(cons.CHARS_LISTBUL[4], iter_start, iter_insert, text_buffer)
+                        elif iter_start.get_char() == cons.CHAR_EQUAL:
+                            dad.special_char_replace(cons.CHARS_LISTBUL[5], iter_start, iter_insert, text_buffer)
+                    elif iter_start.get_char() == cons.CHAR_MINUS and iter_start.backward_char():
+                        if iter_start.get_char() == cons.CHAR_LESSER:
+                            dad.special_char_replace(cons.SPECIAL_CHAR_ARROW_DOUBLE, iter_start, iter_insert, text_buffer)
+                        elif iter_start.get_char() == cons.CHAR_MINUS:
+                            dad.special_char_replace(cons.SPECIAL_CHAR_ARROW_RIGHT, iter_start, iter_insert, text_buffer)
                 elif iter_start.get_char() == cons.CHAR_MINUS and iter_start.backward_char()\
                 and iter_start.get_char() == cons.CHAR_MINUS and iter_start.backward_char()\
                 and iter_start.get_char() == cons.CHAR_LESSER:
@@ -187,16 +273,17 @@ def on_sourceview_event_after_key_press(dad, text_view, event):
                     and iter_start.get_char() == "t" and iter_start.backward_char()\
                     and iter_start.get_char() == cons.CHAR_PARENTH_OPEN:
                         dad.special_char_replace(cons.SPECIAL_CHAR_UNREGISTERED_TRADEMARK, iter_start, iter_insert, text_buffer)
-                # Start bulleted list on "* " at line start
                 elif iter_start.get_char() == cons.CHAR_STAR and iter_start.get_line_offset() == 0:
-                    text_buffer.delete(iter_start, iter_insert)
-                    dad.lists_handler.list_handler(0, text_buffer=text_buffer)
-                # Start todo list on "[]" at line start
-                elif iter_start.get_char() == cons.CHAR_SQ_BR_CLOSE and iter_start.backward_char()\
-                and iter_start.get_char() == cons.CHAR_SQ_BR_OPEN\
-                and iter_start.get_line_offset() == 0:
-                    text_buffer.delete(iter_start, iter_insert)
-                    dad.lists_handler.list_handler(-1, text_buffer=text_buffer)
+                    # Start bulleted list on "* " at line start
+                    dad.special_char_replace(cons.CHARS_LISTBUL[0], iter_start, iter_insert, text_buffer)
+                elif iter_start.get_char() == cons.CHAR_SQ_BR_CLOSE and iter_start.backward_char():
+                    # Start todo list on "[] " at line start
+                    if iter_start.get_line_offset() == 0 and iter_start.get_char() == cons.CHAR_SQ_BR_OPEN:
+                        dad.special_char_replace(cons.CHAR_LISTTODO, iter_start, iter_insert, text_buffer)
+                elif iter_start.get_char() == cons.CHAR_COLON and iter_start.backward_char():
+                    # Start todo list on ":: " at line start
+                    if iter_start.get_line_offset() == 0 and iter_start.get_char() == cons.CHAR_COLON:
+                        dad.special_char_replace(cons.CHARS_LISTBUL[2], iter_start, iter_insert, text_buffer)
     return False
 
 def sourceview_cursor_and_tooltips_handler(dad, text_view, x, y):
@@ -336,11 +423,11 @@ def strip_trailing_spaces(text_buffer):
                 break
     return cleaned_lines
 
-def get_next_chars_from_iter_are(iter_start, num_n_chars):
+def get_next_chars_from_iter_are(iter_start, chars_list):
     """Returns True if the Given Chars are the next 'num' after iter"""
-    for num_n_chars_elem in num_n_chars:
+    for chars in chars_list:
         text_iter = iter_start.copy()
-        (num, chars) = num_n_chars_elem
+        num = len(chars)
         for i in range(num):
             if text_iter.get_char().encode(cons.STR_UTF8) != chars[i]: break
             if i != num-1 and not text_iter.forward_char(): break
@@ -1292,7 +1379,7 @@ def set_bookmarks_menu_items(inst):
             bookmarks_to_rm.append(node_id_str)
             continue
         menu_item = gtk.ImageMenuItem(inst.nodes_names_dict[long(node_id_str)])
-        menu_item.set_image(gtk.image_new_from_stock("cherry_red", gtk.ICON_SIZE_MENU))
+        menu_item.set_image(gtk.image_new_from_stock("pin", gtk.ICON_SIZE_MENU))
         menu_item.connect("activate", select_bookmark_node, node_id_str, inst)
         menu_item.show()
         bookmarks_menu.append(menu_item)
@@ -1385,9 +1472,12 @@ def open_recent_document(menu_item, filepath, dad):
         dad.filepath_open(filepath)
     else:
         dialog_error(_("The Document %s was Not Found") % filepath, dad.window)
-        menu_item.hide()
-        try: dad.recent_docs.remove(filepath)
-        except: pass
+        try:
+            idx = dad.recent_docs.index(filepath)
+            dad.recent_docs.append(dad.recent_docs.pop(idx))
+            set_menu_items_recent_documents(dad)
+        except:
+            pass
 
 def select_bookmark_node(menu_item, node_id_str, dad):
     """Select a Node in the Bookmarks List"""
@@ -1406,11 +1496,10 @@ def bookmarks_handle(dad):
     liststore = gtk.ListStore(str, str, str)
     for node_id_str in dad.bookmarks:
         # icon, node name, node id string
-        liststore.append(["cherry_red", dad.nodes_names_dict[long(node_id_str)], node_id_str])
+        liststore.append(["pin", dad.nodes_names_dict[long(node_id_str)], node_id_str])
     treeview = gtk.TreeView(liststore)
     treeview.set_headers_visible(False)
     treeview.set_reorderable(True)
-    treeview.set_tooltip_text(_("Sort with Drag and Drop, Delete with the Delete Key"))
     treeviewselection = treeview.get_selection()
     def on_key_press_liststore(widget, event):
         keyname = gtk.gdk.keyval_name(event.keyval)
@@ -1440,7 +1529,56 @@ def bookmarks_handle(dad):
     scrolledwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
     scrolledwindow.add(treeview)
     content_area = dialog.get_content_area()
-    content_area.pack_start(scrolledwindow)
+    hbox = gtk.HBox()
+    vbox = gtk.VBox()
+    vbox.set_spacing(1)
+    button_move_up = gtk.Button()
+    button_move_up.set_image(gtk.image_new_from_stock("gtk-go-up", gtk.ICON_SIZE_DND))
+    button_move_down = gtk.Button()
+    button_move_down.set_image(gtk.image_new_from_stock("gtk-go-down", gtk.ICON_SIZE_DND))
+    button_delete = gtk.Button()
+    button_delete.set_image(gtk.image_new_from_stock("gtk-clear", gtk.ICON_SIZE_DND))
+    button_sort_desc = gtk.Button()
+    button_sort_desc.set_image(gtk.image_new_from_stock("gtk-sort-descending", gtk.ICON_SIZE_DND))
+    button_sort_asc = gtk.Button()
+    button_sort_asc.set_image(gtk.image_new_from_stock("gtk-sort-ascending", gtk.ICON_SIZE_DND))
+    def on_button_move_up_clicked(*args):
+        model, tree_iter = treeviewselection.get_selected()
+        if tree_iter:
+            prev_iter = dad.get_tree_iter_prev_sibling(model, tree_iter)
+            if prev_iter:
+                model.swap(tree_iter, prev_iter)
+    button_move_up.connect('clicked', on_button_move_up_clicked)
+    def on_button_move_down_clicked(*args):
+        model, tree_iter = treeviewselection.get_selected()
+        if tree_iter:
+            next_iter = model.iter_next(tree_iter)
+            if next_iter:
+                model.swap(tree_iter, next_iter)
+    button_move_down.connect('clicked', on_button_move_down_clicked)
+    def on_button_delete_clicked(*args):
+        model, tree_iter = treeviewselection.get_selected()
+        if tree_iter:
+            model.remove(tree_iter)
+    button_delete.connect('clicked', on_button_delete_clicked)
+    def on_button_sort_desc_clicked(*args):
+        while dad.node_siblings_sort_iteration(liststore, None, False, 1):
+            pass
+    button_sort_desc.connect('clicked', on_button_sort_desc_clicked)
+    def on_button_sort_asc_clicked(*args):
+        while dad.node_siblings_sort_iteration(liststore, None, True, 1):
+            pass
+    button_sort_asc.connect('clicked', on_button_sort_asc_clicked)
+    vbox.pack_start(button_move_up, expand=False)
+    vbox.pack_start(button_move_down, expand=False)
+    vbox.pack_start(button_delete, expand=False)
+    vbox.pack_start(gtk.Label(), expand=True)
+    vbox.pack_start(button_sort_desc, expand=False)
+    vbox.pack_start(button_sort_asc, expand=False)
+    vbox.pack_start(gtk.Label(), expand=True)
+    hbox.pack_start(scrolledwindow, expand=True)
+    hbox.pack_start(vbox, expand=False)
+    content_area.pack_start(hbox)
     content_area.show_all()
     response = dialog.run()
     temp_bookmarks = []
@@ -1450,7 +1588,17 @@ def bookmarks_handle(dad):
         tree_iter = liststore.iter_next(tree_iter)
     dialog.destroy()
     if response != gtk.RESPONSE_ACCEPT: return False
+    removed_bookmarks = []
+    for old_bookmark in dad.bookmarks:
+        if not old_bookmark in temp_bookmarks:
+            removed_bookmarks.append(old_bookmark)
     dad.bookmarks = temp_bookmarks
+    for removed_bookmark in removed_bookmarks:
+        tree_iter = dad.get_tree_iter_from_node_id(int(removed_bookmark))
+        if tree_iter:
+            dad.update_node_pre_icon(tree_iter)
+            if dad.curr_tree_iter and dad.treestore[tree_iter][3] == dad.treestore[dad.curr_tree_iter][3]:
+                dad.menu_tree_update_for_bookmarked_node(False)
     set_bookmarks_menu_items(dad)
     dad.ctdb_handler.pending_edit_db_bookmarks()
     return True
